# -*- coding: utf-8 -*-
'''
Created on Sat Apr  9 16:17:48 2016

@author: brentshapiro-albert
'''

'''
Last Updated: 4-11-2016

This python file will take .rad files generated through the USB TV-Dongle
receiver for the SRT and convert them to look like the old files generated
using the digital receiver. 

This script can be run from the terminal with the following command:
SRT_File_Converter.py /path/to/file (without the .rad extension)

The output file will be in the same directory as the input file and will be called:
filename_SRTPlot.rad
and can be read into the SRT Plotter java applet.

'''

import csv
import sys
import numpy as np

# This function will open the .rad file we are interested in
def openFile(Filename):
    # We add the .rad extention
    name = Filename + '.rad'
    # Now we open it with csv reader
    with open(name, 'r') as ff:
        reader = csv.reader(ff, skipinitialspace = False, delimiter = ' ')
        # Get rid of empty spaces
        firstLine = []
        writeLine = []
        finalArray = []
        # Set null values
        source2 = 'null'
        az1 = 0
        el1 = 0
        offcheck = [0.0, 0.0]
        offaz = 1.0
        offel = 1.0
        offset = False
        for row in reader:
            np.array(row)
            for val in row:
                if val == '':
                    row.remove(val)
            # Now we take just the values we need from the first two lines
            '''
            We make an empty list to put the values we want in:
            Date, az, el, offset az, offset el, fstart, spacing, 1, nchannels
            '''
            if row[0] == 'DATE':
                # Get rid of leftover white space
                k = 0
                while k < len(row):
                    if row[k] == '':
                        del row[k]
                    k += 1
                # get date
                firstLine.append(row[1])
                # get az and el
                az = float(row[5])
                firstLine.append(float(row[5]))
                el = float(row[7])
                firstLine.append(float(row[7]))
                # Get the source 
                source = row[21]
                # still empty space don't know why...
                # Not a solution, but works for now
                if source == 'source':
                    source = row[22]
                # Check to see if there is an offset
                if (az1 != az or el1 != el) and source2 == source:
                    offset = False
                    if offaz == offcheck[0] or offel == offcheck[1]:
                        offaz = -1*offaz
                        offel = -1*offel
                # Check the offset and add value
                # This is complicated. Not sure I totally understand whats
                # going on. Seems to work though
                if offset == False:
                    if (az1 != az or el1 != el) and source2 == source:
                        if offaz != -1*offcheck[0] or offel != -1*offcheck[1]:
                            offaz = az - az1
                            offel = el - el1
                            firstLine.append(offaz)
                            firstLine.append(offel)
                            offset = True
                        else:
                            firstLine.append(0.0)
                            firstLine.append(0.0)
                    else:
                        firstLine.append(0.0)
                        firstLine.append(0.0)
                else:
                    if offcheck[0] == -1*offaz:
                        firstLine.append(0.0)
                        offaz = 0.0
                    if offcheck[1] == -1*offel:
                        firstLine.append(0.0)
                        offel = 0.0
                    else:
                        firstLine.append(offaz)
                        firstLine.append(offel)
                    offcheck = [offaz, offel]
            # Get next set of values
            elif row[0] == 'Fstart':
                 # Get rid of leftover white space
                k = 0
                while k < len(row):
                    if row[k] == '':
                        del row[k]
                    k += 1
                # Get Fstart
                firstLine.append(float(row[1]))
                # Get spacing
                firstLine.append(float(row[5]))
                # add a 1 (not sure why, do need though)
                firstLine.append(1)
                # Get channels
                firstLine.append(int(row[16]))
            # Now we can reset the first line
            elif row[0] == 'Spectrum':
                # We add the source as well
                if source != source2:
                    # Sets source not at stow
                    if source != 'at_stow':
                        source2 = source
                        source = '* ' + source
                        writeLine.insert(0, source)
                    # sets source if not at stow but not at pre-programmed source
                    elif source == 'at_stow' and firstLine[2] != -2.0:
                        source2 = source
                        source = '* az ' + str(firstLine[1]) + ' el ' + str(firstLine[2])
                        writeLine.insert(0, source)
                    elif source == 'at_stow' and firstLine[1] != 21.0:
                        source2 = source
                        source = '* az ' + str(firstLine[1]) + ' el ' + str(firstLine[2])
                        writeLine.insert(0, source)
                    # set source as at_stow
                    else:
                        if source2 != '* at_stow':
                            source = '* ' + source
                            source2 = source
                            writeLine.insert(0, source)
            # Now we go through and take the data and put it into a list
            else:
                # Add data to the same line
                for val in row:
                    if val != '':
                        val = float(val)
                        firstLine.append(val)
                # Now we add the data to our list
                writeLine.append(firstLine)
                # Now we add the line to a final array
                finalArray.append(writeLine)
                # We save the old az and el values for the offset
                az1 = az
                el1 = el
                # Now we clear writeLine
                firstLine = []
                writeLine = []
        # Now we close the file
        ff.close()
    # Now we return the final array
    return finalArray
    
# Now we write a function that takes this output and writes it to a new
# .rad file in the correct format
def writeFile(finalArray, Filename):
    # We make the file name a .rad file and add an extention on so as not to
    # delete the orginal
    Filename = Filename + '_SRTPlot.rad'
    # Now we create it with a csv writer
    with open(Filename, 'wb') as ff:
        # Got rid of quotes, still adding extra spaces...
        radWriter = csv.writer(ff, delimiter = ' ', escapechar = ' ', \
        quoting = csv.QUOTE_NONE)
        # Now we go through our array and write it line by line
        k = 0
        while k < len(finalArray):
            # Open one chunk at a time (source, file info, and data)
            chunk = finalArray[k]
            # Write one line at a time
            for row in chunk:
                # Gets rid of extra spaces in source title
                if row[0] == '*':
                    ff.write(row + '\n')
                # writes all other lines of numbers
                else:
                    radWriter.writerow(row)
            # Go to next chunk
            k += 1
        ff.close()
            
            
# Now we write just a main function to call once
def main(Filename):               
    finalArray = openFile(Filename)
    writeFile(finalArray, Filename)
    
# Testing function
#start_time = time.time()
Filename = str(sys.argv[1])
main(Filename)
#print 'Time running code: %f seconds' % (time.time()-start_time)
